package com.aidlab.aidlab_sdk

import com.aidlab.sdk.*
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger

fun Device.toFlutter(): Device_ {

    return Device_(this.firmwareRevision, this.hardwareRevision, this.serialNumber, this.name(), this.manufacturerName, this.address())
}

class AidlabManagerHandler: AidlabManager_, AidlabManagerDelegate {

    private var aidlabManager: AidlabManager? = null
    
    private lateinit var binaryMessenger: BinaryMessenger

    private var delegate: AidlabManagerDelegate_? = null

    private var connectedDevice: DeviceHandler? = null

    fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {

        aidlabManager = AidlabManager(binding.applicationContext, this)
        binaryMessenger = binding.binaryMessenger

        delegate = AidlabManagerDelegate_(binding.binaryMessenger)
        AidlabManager_.setUp(binding.binaryMessenger, this)
    }

    override fun scan(mode: ScanMode_) {
        aidlabManager?.scan()
    }

    override fun stopScan() {
        aidlabManager?.stopScan()
    }

    override fun connect(device: Device_) {
        connectedDevice = DeviceHandler(binaryMessenger, device)
        discoveredDevices[device.address]?.connect(connectedDevice!!)
    }

    override fun collect(
        device: Device_,
        dataTypes: List<FlutterDataType_>,
        dataTypesToStore: List<FlutterDataType_>
    ) {
        connectedDevice?.collect(dataTypes, dataTypesToStore)
    }

    override fun disconnect(device: Device_) {
        connectedDevice?.disconnect()
    }

    /// Map of discovered devices: address: Device
    private var discoveredDevices: MutableMap<String, Device> = mutableMapOf()

    override fun didDiscover(device: Device, rssi: Int) {
        discoveredDevices[device.address()] = device
        delegate?.didDiscover(device.toFlutter(), rssi.toLong()) {}
    }

    override fun onDeviceScanFailed(errorCode: Int) {
        delegate?.onScanFailed(errorCode.toLong()) {}
    }

    override fun onDeviceScanStarted() {
        delegate?.onDeviceScanStarted {}
    }

    override fun onDeviceScanStopped() {
        delegate?.onDeviceScanStopped {}
    }
}