package com.aidlab.aidlab_sdk

import com.aidlab.sdk.*
import io.flutter.Log
import io.flutter.plugin.common.BinaryMessenger
import java.util.EnumSet

class DeviceHandler(
    binaryMessenger: BinaryMessenger,
    deviceAsFlutter: Device_
) : DeviceDelegate {

    private var delegate: DeviceDelegate_? = null

    private var device: Device? = null
    private var deviceAsFlutter: Device_

    init {
        delegate = DeviceDelegate_(binaryMessenger)
        this.deviceAsFlutter = deviceAsFlutter
    }

    fun collect(dataTypes: List<FlutterDataType_>,
                dataTypesToStore: List<FlutterDataType_>) {

        val kotlinDataTypes = EnumSet.noneOf(DataType::class.java)
        dataTypes.forEach { flutterType ->
            val kotlinType = DataType.entries.firstOrNull { it.value == flutterType.signal?.ordinal }
            kotlinType?.let { kotlinDataTypes.add(it) }
        }

        val kotlinDataTypesToStore = EnumSet.noneOf(DataType::class.java)
        dataTypesToStore.forEach { flutterType ->
            val kotlinType = DataType.entries.firstOrNull { it.value == flutterType.signal?.ordinal }
            kotlinType?.let { kotlinDataTypesToStore.add(it) }
        }

        device?.collect(kotlinDataTypes, kotlinDataTypesToStore)
    }

    fun disconnect() {
        device?.disconnect()
    }

    override fun didConnect(device: Device) {
        this.deviceAsFlutter = device.toFlutter()
        this.device = device
        delegate?.didConnect(this.deviceAsFlutter) {}
    }

    override fun didDisconnect(device: Device, disconnectReason: DisconnectReason) {

        val reason = DisconnectReason_.ofRaw(disconnectReason.value)!!
        delegate?.didDisconnect(this.deviceAsFlutter, reason) {}
    }

    override fun didReceiveECG(device: Device, timestamp: Long, value: Float) {
        delegate?.didReceiveECG(this.deviceAsFlutter, timestamp, value.toDouble() ) {}
    }

    override fun didDetectExercise(device: Device, exercise: Exercise) {
        val exercise_ = Exercise_.ofRaw(exercise.value)
        delegate?.didDetectExercise(this.deviceAsFlutter, exercise_!!) {}
    }

    override fun didDetectPastUserEvent(timestamp: Long) {
        delegate?.didDetectPastUserEvent(this.deviceAsFlutter, timestamp) {}
    }

    override fun didReceiveAccelerometer(
        device: Device,
        timestamp: Long,
        ax: Float,
        ay: Float,
        az: Float
    ) {
        delegate?.didReceiveAccelerometer(this.deviceAsFlutter, timestamp, ax.toDouble(), ay.toDouble(), az.toDouble()) {}
    }

    override fun didReceiveActivity(device: Device, timestamp: Long, activity: ActivityType) {
        val activity_ = ActivityType_.ofRaw(activity.value)
        if(activity_ != null) {
            delegate?.didReceiveActivity(this.deviceAsFlutter, timestamp, activity_) {}
        }
    }

    override fun didReceiveBatteryLevel(device: Device, stateOfCharge: Int) {
        delegate?.didReceiveBatteryLevel(this.deviceAsFlutter, stateOfCharge.toLong()) {}
    }

    override fun didReceiveBodyPosition(
        device: Device,
        timestamp: Long,
        bodyPosition: BodyPosition
    ) {
        val body = BodyPosition_.ofRaw(bodyPosition.value)
        delegate?.didReceiveBodyPosition(this.deviceAsFlutter, timestamp, body!!) {}
    }

    override fun didReceiveCommand(device: Device) {
        Log.d("AidlabHandler", "didReceiveCommand")
        delegate?.didReceiveCommand(this.deviceAsFlutter) {}
    }

    override fun didReceiveError(error: String) {
        delegate?.didReceiveError(error) {}
    }

    override fun didReceiveGyroscope(
        device: Device,
        timestamp: Long,
        qx: Float,
        qy: Float,
        qz: Float
    ) {
        delegate?.didReceiveGyroscope(this.deviceAsFlutter, timestamp, qx.toDouble(), qy.toDouble(), qz.toDouble()) {}
    }

    override fun didReceiveHeartRate(device: Device, timestamp: Long, heartRate: Int) {
        delegate?.didReceiveHeartRate(this.deviceAsFlutter, timestamp, heartRate.toLong()) {}
    }

    override fun didReceiveRr(device: Device, timestamp: Long, rr: Int) {
        delegate?.didReceiveRr(this.deviceAsFlutter, timestamp, rr.toLong()) {}
    }

    override fun didReceiveMagnetometer(
        device: Device,
        timestamp: Long,
        mx: Float,
        my: Float,
        mz: Float
    ) {
        delegate?.didReceiveMagnetometer(this.deviceAsFlutter, timestamp, mx.toDouble(), my.toDouble(), mz.toDouble()) {}
    }

    override fun didReceiveMessage(device: Device, process: String, message: String) {
        delegate?.didReceiveMessage(this.deviceAsFlutter, process, message) {}
    }

    override fun didReceiveOrientation(
        device: Device, timestamp: Long,
        roll: Float, pitch: Float, yaw: Float
    ) {
        delegate?.didReceiveOrientation(this.deviceAsFlutter, timestamp, roll.toDouble(), pitch.toDouble(), yaw.toDouble()) {}
    }

    override fun didReceivePastAccelerometer(
        device: Device,
        timestamp: Long,
        ax: Float,
        ay: Float,
        az: Float
    ) {
        delegate?.didReceivePastAccelerometer(this.deviceAsFlutter, timestamp, ax.toDouble(), ay.toDouble(), az.toDouble()) {}
    }

    override fun didReceivePastActivity(device: Device, timestamp: Long, activity: ActivityType) {
        delegate?.didReceivePastActivity(this.deviceAsFlutter, timestamp, ActivityType_.ofRaw(activity.value)!!) {}
    }

    override fun didReceivePastBodyPosition(
        device: Device,
        timestamp: Long,
        bodyPosition: BodyPosition
    ) {
        delegate?.didReceivePastBodyPosition(this.deviceAsFlutter, timestamp, BodyPosition_.ofRaw(bodyPosition.value)!!) {}
    }

    override fun didReceivePastECG(device: Device, timestamp: Long, value: Float) {
        delegate?.didReceivePastECG(this.deviceAsFlutter, timestamp, value.toDouble()) {}
    }

    override fun didReceivePastGyroscope(
        device: Device,
        timestamp: Long,
        qx: Float,
        qy: Float,
        qz: Float
    ) {
        delegate?.didReceivePastGyroscope(this.deviceAsFlutter, timestamp, qx.toDouble(), qy.toDouble(), qz.toDouble()) {}
    }

    override fun didReceivePastHeartRate(device: Device, timestamp: Long, heartRate: Int) {
        delegate?.didReceivePastHeartRate(this.deviceAsFlutter, timestamp, heartRate.toLong()) {}
    }

    override fun didReceivePastMagnetometer(
        device: Device,
        timestamp: Long,
        mx: Float,
        my: Float,
        mz: Float
    ) {
        delegate?.didReceivePastMagnetometer(this.deviceAsFlutter, timestamp, mx.toDouble(), my.toDouble(), mz.toDouble()) {}
    }

    override fun didReceivePastOrientation(
        device: Device,
        timestamp: Long,
        roll: Float,
        pitch: Float,
        yaw: Float
    ) {
        delegate?.didReceivePastOrientation(this.deviceAsFlutter, timestamp, roll.toDouble(), pitch.toDouble(), yaw.toDouble()) {}
    }

    override fun didReceivePastPressure(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceivePastPressure(this.deviceAsFlutter, timestamp, value.toLong() ) {}
    }

    override fun didReceivePastQuaternion(
        device: Device,
        timestamp: Long,
        qw: Float,
        qx: Float,
        qy: Float,
        qz: Float
    ) {
        delegate?.didReceivePastQuaternion(this.deviceAsFlutter, timestamp, qw.toDouble(), qx.toDouble(), qy.toDouble(), qz.toDouble()) {}
    }

    override fun didReceivePastRespiration(device: Device, timestamp: Long, value: Float) {
        delegate?.didReceivePastRespiration(this.deviceAsFlutter, timestamp, value.toDouble() ) {}
    }

    override fun didReceivePastRespirationRate(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceivePastRespirationRate(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceivePastRr(device: Device, timestamp: Long, rr: Int) {
        delegate?.didReceivePastRr(this.deviceAsFlutter, timestamp, rr.toLong()) {}
    }

    override fun didReceivePastSignalQuality(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceivePastSignalQuality(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceivePastSkinTemperature(device: Device, timestamp: Long, value: Float) {
        delegate?.didReceivePastSkinTemperature(this.deviceAsFlutter, timestamp, value.toDouble()) {}
    }

    override fun didReceivePastSoundFeatures(device: Device, timestamp: Long, values: FloatArray) {
        delegate?.didReceivePastSoundFeatures(this.deviceAsFlutter, timestamp, values.map { it.toDouble() } ) {}
    }

    override fun didReceivePastSoundVolume(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceivePastSoundVolume(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceivePastSteps(device: Device, timestamp: Long, value: Long) {
        delegate?.didReceivePastSteps(this.deviceAsFlutter, timestamp, value) {}
    }

    override fun didReceivePressure(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceivePressure(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceiveQuaternion(
        device: Device,
        timestamp: Long,
        qw: Float,
        qx: Float,
        qy: Float,
        qz: Float
    ) {
        delegate?.didReceiveQuaternion(this.deviceAsFlutter, timestamp, qw.toDouble(), qx.toDouble(), qy.toDouble(), qz.toDouble()) {}
    }

    override fun didReceiveRespiration(device: Device, timestamp: Long, value: Float) {
        delegate?.didReceiveRespiration(this.deviceAsFlutter, timestamp, value.toDouble() ) {}
    }

    override fun didReceiveRespirationRate(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceiveRespirationRate(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceiveSignalQuality(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceiveSignalQuality(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceiveSkinTemperature(device: Device, timestamp: Long, value: Float) {
        delegate?.didReceiveSkinTemperature(this.deviceAsFlutter, timestamp, value.toDouble()) {}
    }

    override fun didReceiveSoundFeatures(device: Device, timestamp: Long, values: FloatArray) {
        delegate?.didReceiveSoundFeatures(this.deviceAsFlutter, timestamp, values.map { it.toDouble() }) {}
    }

    override fun didReceiveSoundVolume(device: Device, timestamp: Long, value: Int) {
        delegate?.didReceiveSoundVolume(this.deviceAsFlutter, timestamp, value.toLong()) {}
    }

    override fun didReceiveSteps(device: Device, timestamp: Long, value: Long) {
        delegate?.didReceiveSteps(this.deviceAsFlutter, timestamp, value) {}
    }

    override fun didReceiveUnsynchronizedSize(
        device: Device,
        unsynchronizedSize: Int,
        syncBytesPerSecond: Float
    ) {
        delegate?.didReceiveUnsynchronizedSize(this.deviceAsFlutter, unsynchronizedSize.toLong(), syncBytesPerSecond.toDouble()) {}
    }

    override fun syncStateDidChange(device: Device, state: SyncState) {
        val sync = SyncState_.ofRaw(state.value)
        delegate?.syncStateDidChange(this.deviceAsFlutter, sync!!) {}
    }

    override fun wearStateDidChange(device: Device, wearState: WearState) {
        val wear = WearState_.ofRaw(wearState.value)
        delegate?.wearStateDidChange(this.deviceAsFlutter, wear!!) {}
    }
}