//
//  DevicedeviceDelegate_.swift
//  aidlab_sdk
//
//  Created by Jakub Domaszewicz on 01/03/2024.
//

import Foundation
import Flutter

extension Device {

    func toFlutter() -> Device_ {

        return Device_(firmwareRevision: self.firmwareRevision, hardwareRevision: self.hardwareRevision, serialNumber: self.serialNumber, name: self.name, manufacturerName: self.manufacturerName, address: self.address.uuidString)
    }
}

public class DeviceHandler: DeviceDelegate {

    private let deviceDelegate_: DeviceDelegate_

    /// Prevents hammering as `toFlutter()` is too costly
    private var device_: Device_

    private var device: Device

    init(messenger: FlutterBinaryMessenger, device: Device) {
        self.deviceDelegate_ = DeviceDelegate_(binaryMessenger: messenger)
        self.device = device
        self.device_ = device.toFlutter()
    }

    private func mapActivity(_ value: Int) -> Int {
        let mappingDict: [Int: Int] = [0: 0, 1: 1, 2: 2, 4: 3, 8: 4, 16: 5]
        return mappingDict[value, default: 0]
    }

    public func connect() {

        device.connect(delegate: self)
    }

    public func disconnect() {

        device.disconnect()
    }

    public func didConnect(_ device: Device) {
        self.device_ = device.toFlutter()
        deviceDelegate_.didConnect(device: self.device_, completion: { _ in })
    }

    public func didReceiveError(_ device: Device, error: Error) {

        deviceDelegate_.didReceiveError(error: error.localizedDescription, completion: { _ in })
    }

    public func didDisconnect(_ device: Device, reason: DisconnectReason) {

        deviceDelegate_.didDisconnect(device: self.device_, reason: DisconnectReason_(rawValue: reason.rawValue)!, completion: { _ in })
    }

    public func didUpdateRSSI(_ device: Device, rssi: Int32) {

        /// TODO
    }

    public func didReceiveECG(_ device: Device, timestamp: UInt64, value: Float) {

        deviceDelegate_.didReceiveECG(device: self.device_, timestamp: Int64(timestamp), value: Double(value), completion: { _ in })
    }

    public func didReceiveRespiration(_ device: Device, timestamp: UInt64, value: Float) {

        deviceDelegate_.didReceiveRespiration(device: self.device_, timestamp: Int64(timestamp), value: Double(value), completion: { _ in })
    }

    public func didReceiveBatteryLevel(_ device: Device, stateOfCharge: UInt8) {

        deviceDelegate_.didReceiveBatteryLevel(device: self.device_, stateOfCharge: Int64(stateOfCharge), completion: { _ in })
    }

    public func didReceiveSteps(_ device: Device, timestamp: UInt64, value: UInt64) {

        deviceDelegate_.didReceiveSteps(device: self.device_, timestamp: Int64(timestamp), steps: Int64(value), completion: { _ in })
    }

    public func didReceiveSkinTemperature(_ device: Device, timestamp: UInt64, value: Float) {

        deviceDelegate_.didReceiveSkinTemperature(device: self.device_, timestamp: Int64(timestamp), value: Double(value), completion: { _ in })
    }

    public func didReceiveAccelerometer(_ device: Device, timestamp: UInt64, ax: Float, ay: Float, az: Float) {

        deviceDelegate_.didReceiveAccelerometer(device: self.device_, timestamp: Int64(timestamp), ax: Double(ax), ay: Double(ay), az: Double(az), completion: { _ in })
    }

    public func didReceiveGyroscope(_ device: Device, timestamp: UInt64, qx: Float, qy: Float, qz: Float) {

        deviceDelegate_.didReceiveGyroscope(device: self.device_, timestamp: Int64(timestamp), qx: Double(qx), qy: Double(qy), qz: Double(qz), completion: { _ in })
    }

    public func didReceiveMagnetometer(_ device: Device, timestamp: UInt64, mx: Float, my: Float, mz: Float) {

        deviceDelegate_.didReceiveMagnetometer(device: self.device_, timestamp: Int64(timestamp), mx: Double(mx), my: Double(my), mz: Double(mz), completion: { _ in })
    }

    public func didReceiveQuaternion(_ device: Device, timestamp: UInt64, qw: Float, qx: Float, qy: Float, qz: Float) {

        deviceDelegate_.didReceiveQuaternion(device: self.device_, timestamp: Int64(timestamp), qw: Double(qw), qx: Double(qx), qy: Double(qy), qz: Double(qz), completion: { _ in })
    }

    public func didReceiveOrientation(_ device: Device, timestamp: UInt64, roll: Float, pitch: Float, yaw: Float) {

        deviceDelegate_.didReceiveOrientation(device: self.device_, timestamp: Int64(timestamp), roll: Double(roll), pitch: Double(pitch), yaw: Double(yaw), completion: { _ in })
    }

    public func didReceiveBodyPosition(_ device: Device, timestamp: UInt64, bodyPosition: BodyPosition) {

        deviceDelegate_.didReceiveBodyPosition(device: self.device_, timestamp: Int64(timestamp), bodyPosition: BodyPosition_(rawValue: bodyPosition.rawValue)!, completion: { _ in })
    }

    public func didReceiveHeartRate(_ device: Device, timestamp: UInt64, heartRate: Int32) {

        deviceDelegate_.didReceiveHeartRate(device: self.device_, timestamp: Int64(timestamp), heartRate: Int64(heartRate), completion: { _ in })
    }

    public func didReceiveRr(_ device: Device, timestamp: UInt64, rr: Int32) {

        deviceDelegate_.didReceiveRr(device: self.device_, timestamp: Int64(timestamp), rr: Int64(rr), completion: { _ in })
    }

    public func didReceiveRespirationRate(_ device: Device, timestamp: UInt64, value: UInt32) {

        deviceDelegate_.didReceiveRespirationRate(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }

    public func didReceiveSoundVolume(_ device: Device, timestamp: UInt64, soundVolume: UInt16) {

        deviceDelegate_.didReceiveSoundVolume(device: self.device_, timestamp: Int64(timestamp), value: Int64(soundVolume), completion: { _ in })
    }

    public func didDetect(_ device: Device, exercise: Exercise) {

        deviceDelegate_.didDetectExercise(device: self.device_, exercise: Exercise_(rawValue: exercise.rawValue)!, completion: { _ in })
    }

    public func didDetect(_ device: Device, timestamp: UInt64, activity: ActivityType) {
        
        guard let flutterActivityType = ActivityType_(rawValue: activity.rawValue) else {
            didReceiveError(device, error: AidlabError(message: "Incorrect activity: \(activity.rawValue)"))
            return
        }
        deviceDelegate_.didReceiveActivity(device: self.device_, timestamp: Int64(timestamp), activity: flutterActivityType, completion: { _ in })
    }

    public func wearStateDidChange(_ device: Device, state: WearState) {

        deviceDelegate_.wearStateDidChange(device: self.device_, wearState: WearState_(rawValue: state.rawValue)!, completion: { _ in })
    }

    public func didReceiveCommand(_ device: Device) {

        deviceDelegate_.didReceiveCommand(device: self.device_, completion: { _ in })
    }

    public func didReceiveMessage(_ device: Device, process: String, message: String) {

        deviceDelegate_.didReceiveMessage(device: self.device_, process: process, message: message, completion: { _ in })
    }

    public func didDetectUserEvent(_ device: Device, timestamp: UInt64) {

        deviceDelegate_.didDetectUserEvent(device: self.device_, timestamp: Int64(timestamp), completion: { _ in })
    }

    public func didReceiveSignalQuality(_ timestamp: UInt64, value: Int32) {

        deviceDelegate_.didReceiveSignalQuality(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }

    public func syncStateDidChange(_ device: Device, state: SyncState) {

        deviceDelegate_.syncStateDidChange(device: self.device_, state: SyncState_(rawValue: state.rawValue)!, completion: { _ in })
    }

    // -- Past

    public func didReceivePastECG(_ device: Device, timestamp: UInt64, value: Float) {

        deviceDelegate_.didReceivePastECG(device: self.device_, timestamp: Int64(timestamp), value: Double(value), completion: { _ in })
    }

    public func didReceivePastRespiration(_ device: Device, timestamp: UInt64, value: Float) {

        deviceDelegate_.didReceivePastRespiration(device: self.device_, timestamp: Int64(timestamp), value: Double(value), completion: { _ in })
    }

    public func didReceivePastSkinTemperature(_ device: Device, timestamp: UInt64, value: Float) {

        deviceDelegate_.didReceivePastSkinTemperature(device: self.device_, timestamp: Int64(timestamp), value: Double(value), completion: { _ in })
    }

    public func didReceivePastHeartRate(_ device: Device, timestamp: UInt64, heartRate: Int32) {

        deviceDelegate_.didReceivePastHeartRate(device: self.device_, timestamp: Int64(timestamp), heartRate: Int64(heartRate), completion: { _ in })
    }

    public func didReceivePastRr(_ device: Device, timestamp: UInt64, rr: Int32) {
            
        deviceDelegate_.didReceivePastRr(device: self.device_, timestamp: Int64(timestamp), rr: Int64(rr), completion: { _ in })
    }

    public func didReceiveUnsynchronizedSize(_ device: Device, unsynchronizedSize: UInt32, syncBytesPerSecond: Float) {

        deviceDelegate_.didReceiveUnsynchronizedSize(device: self.device_, unsynchronizedSize: Int64(unsynchronizedSize), syncBytesPerSecond: Double(syncBytesPerSecond), completion: { _ in })
    }

    public func didReceivePastRespirationRate(_ device: Device, timestamp: UInt64, value: UInt32) {

        deviceDelegate_.didReceivePastRespirationRate(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }

    public func didReceivePressure(_ device: Device, timestamp: UInt64, value: Int32) {

        deviceDelegate_.didReceivePressure(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }

    public func pressureWearStateDidChange(_ device: Device, wearState: WearState) {
        
        deviceDelegate_.pressureWearStateDidChange(device: self.device_, wearState: WearState_(rawValue: wearState.rawValue)!, completion: { _ in })
    }

    public func didReceivePastActivity(_ device: Device, timestamp: UInt64, activity: ActivityType) {

        guard let flutterActivityType = ActivityType_(rawValue: mapActivity(activity.rawValue)) else {
            didReceiveError(device, error: AidlabError(message: "Incorrect activity: \(activity.rawValue)"))
            return
        }
        deviceDelegate_.didReceivePastActivity(device: self.device_, timestamp: Int64(timestamp), activity: flutterActivityType, completion: { _ in })
    }

    public func didReceivePastSteps(_ device: Device, timestamp: UInt64, value: UInt64) {

        deviceDelegate_.didReceivePastSteps(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }

    public func didReceivePastSoundVolume(_ device: Device, timestamp: UInt64, soundVolume: UInt16) {

        deviceDelegate_.didReceivePastSoundVolume(device: self.device_, timestamp: Int64(timestamp), soundVolume: Int64(soundVolume), completion: { _ in })
    }

    public func didReceivePastAccelerometer(_ device: Device, timestamp: UInt64, ax: Float, ay: Float, az: Float) {

        deviceDelegate_.didReceivePastAccelerometer(device: self.device_, timestamp: Int64(timestamp), ax: Double(ax), ay: Double(ay), az: Double(az), completion: { _ in })
    }

    public func didReceivePastGyroscope(_ device: Device, timestamp: UInt64, qx: Float, qy: Float, qz: Float) {

        deviceDelegate_.didReceivePastGyroscope(device: self.device_, timestamp: Int64(timestamp), qx: Double(qx), qy: Double(qy), qz: Double(qz), completion: { _ in })
    }

    public func didReceivePastMagnetometer(_ device: Device, timestamp: UInt64, mx: Float, my: Float, mz: Float) {

        deviceDelegate_.didReceivePastMagnetometer(device: self.device_, timestamp: Int64(timestamp), mx: Double(mx), my: Double(my), mz: Double(mz), completion: { _ in })
    }

    public func didReceivePastQuaternion(_ device: Device, timestamp: UInt64, qw: Float, qx: Float, qy: Float, qz: Float) {

        deviceDelegate_.didReceivePastQuaternion(device: self.device_, timestamp: Int64(timestamp), qw: Double(qw), qx: Double(qx), qy: Double(qy), qz: Double(qz), completion: { _ in })
    }

    public func didReceivePastOrientation(_ device: Device, timestamp: UInt64, roll: Float, pitch: Float, yaw: Float) {

        deviceDelegate_.didReceivePastOrientation(device: self.device_, timestamp: Int64(timestamp), roll: Double(roll), pitch: Double(pitch), yaw: Double(yaw), completion: { _ in })
    }

    public func didReceivePastBodyPosition(_ device: Device, timestamp: UInt64, bodyPosition: BodyPosition) {

        deviceDelegate_.didReceivePastBodyPosition(device: self.device_, timestamp: Int64(timestamp), bodyPosition: BodyPosition_(rawValue: bodyPosition.rawValue)!, completion: { _ in })
    }

    public func didReceivePastPressure(_ device: Device, timestamp: UInt64, value: Int32) {

        deviceDelegate_.didReceivePastPressure(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }

    public func didDetectPastUserEvent(_ device: Device, timestamp: UInt64) {

        deviceDelegate_.didDetectPastUserEvent(device: self.device_, timestamp: Int64(timestamp), completion: { _ in })
    }

    public func didReceivePastSignalQuality(_ device: Device, timestamp: UInt64, value: UInt8) {

        deviceDelegate_.didReceivePastSignalQuality(device: self.device_, timestamp: Int64(timestamp), value: Int64(value), completion: { _ in })
    }
}
