import 'package:aidlab_sdk/src/platform_aidlab_sdk.dart' as sdk;

typedef DataType = sdk.DataType_;
typedef DisconnectReason = sdk.DisconnectReason_;
typedef Exercise = sdk.Exercise_;
typedef ScanMode = sdk.ScanMode_;
typedef SyncState = sdk.SyncState_;
typedef WearState = sdk.WearState_;
typedef BodyPosition = sdk.BodyPosition_;
typedef ActivityType = sdk.ActivityType_;

abstract class AidlabManagerDelegate {
  void onBluetoothStarted();

  void onDeviceScanStarted();

  void onDeviceScanStopped();

  void onScanFailed(int errorCode);

  void didDiscover(Device device, int rssi);
}

class AidlabManager implements sdk.AidlabManagerDelegate_ {
  sdk.AidlabManager_ aidlabManager_ = sdk.AidlabManager_();
  AidlabManagerDelegate? delegate;

  AidlabManager(AidlabManagerDelegate delegate) {
    sdk.AidlabManagerDelegate_.setup(this);
    this.delegate = delegate;
  }

  void scan(ScanMode mode) {
    aidlabManager_.scan(mode);
  }

  void stopScan() {
    aidlabManager_.stopScan();
  }

  void dispose() {
    sdk.AidlabManagerDelegate_.setup(null);
  }

  @override
  void didDiscover(sdk.Device_ device, int rssi) {
    delegate?.didDiscover(Device(device, aidlabManager_), rssi);
  }

  @override
  void onBluetoothStarted() {
    delegate?.onBluetoothStarted();
  }

  @override
  void onDeviceScanStarted() {
    delegate?.onDeviceScanStarted();
  }

  @override
  void onDeviceScanStopped() {
    delegate?.onDeviceScanStopped();
  }

  @override
  void onScanFailed(int errorCode) {
    delegate?.onScanFailed(errorCode);
  }
}

class Device implements sdk.DeviceDelegate_ {
  sdk.Device_? device_;
  sdk.AidlabManager_? aidlabManager_;
  DeviceDelegate? delegate;

  String address = '';
  String? name;
  String? firmwareRevision;
  String? hardwareRevision;
  String? manufacturerName;
  String? serialNumber;

  Device(sdk.Device_ device_, sdk.AidlabManager_ aidlabManager_) {
    this.device_ = device_;
    this.aidlabManager_ = aidlabManager_;
    this.name = device_.name ?? '';
    this.address = device_.address;

    this.firmwareRevision = device_.firmwareRevision;
    this.hardwareRevision = device_.hardwareRevision;
    this.manufacturerName = device_.manufacturerName;
    this.serialNumber = device_.serialNumber;
  }

  void connect(DeviceDelegate delegate) {
    this.delegate = delegate;
    sdk.DeviceDelegate_.setup(this);
    aidlabManager_!.connect(device_!);
  }

  void disconnect() {
    aidlabManager_?.disconnect(device_!);
  }

  void collect(List<DataType> dataTypes, List<DataType> dataTypesToStore) {
    aidlabManager_!.collect(
        device_!,
        dataTypes.map((e) => sdk.FlutterDataType_()..signal = e).toList(),
        dataTypesToStore
            .map((e) => sdk.FlutterDataType_()..signal = e)
            .toList());
  }

  void dispose() {
    sdk.DeviceDelegate_.setup(null);
  }

  @override
  void didConnect(sdk.Device_ device_) {
    this.firmwareRevision = device_.firmwareRevision;
    this.hardwareRevision = device_.hardwareRevision;
    this.manufacturerName = device_.manufacturerName;
    this.serialNumber = device_.serialNumber;

    this.delegate!.didConnect(this);
  }

  @override
  void didDetectExercise(sdk.Device_ device, sdk.Exercise_ exercise) {
    delegate?.didDetectExercise(this, exercise);
  }

  @override
  void didDetectPastUserEvent(sdk.Device_ device, int timestamp) {
    delegate?.didReceivePastUserEvent(this, timestamp);
  }

  @override
  void didDetectUserEvent(sdk.Device_ device, int timestamp) {
    delegate?.didDetectUserEvent(this, timestamp);
  }

  @override
  void didDisconnect(sdk.Device_? device, sdk.DisconnectReason_ reason) {
    delegate?.didDisconnect(this, reason);
  }

  @override
  void didReceiveAccelerometer(
      sdk.Device_ device, int timestamp, double ax, double ay, double az) {
    delegate?.didReceiveAccelerometer(this, timestamp, ax, ay, az);
  }

  @override
  void didReceiveActivity(
      sdk.Device_ device, int timestamp, sdk.ActivityType_ activity) {
    delegate?.didReceiveActivity(this, timestamp, activity);
  }

  @override
  void didReceiveBatteryLevel(sdk.Device_ device, int stateOfCharge) {
    delegate?.didReceiveBatteryLevel(this, stateOfCharge);
  }

  @override
  void didReceiveBodyPosition(
      sdk.Device_ device, int timestamp, sdk.BodyPosition_ bodyPosition) {
    delegate?.didReceiveBodyPosition(this, timestamp, bodyPosition);
  }

  @override
  void didReceiveCommand(sdk.Device_ device) {
    delegate?.didReceiveCommand(this);
  }

  @override
  void didReceiveECG(sdk.Device_ device, int timestamp, double value) {
    delegate?.didReceiveECG(this, timestamp, value);
  }

  @override
  void didReceiveError(String error) {
    delegate?.didReceiveError(error);
  }

  @override
  void didReceiveGyroscope(
      sdk.Device_ device, int timestamp, double qx, double qy, double qz) {
    delegate?.didReceiveGyroscope(this, timestamp, qx, qy, qz);
  }

  @override
  void didReceiveHeartRate(sdk.Device_ device, int timestamp, int heartRate) {
    delegate?.didReceiveHeartRate(this, timestamp, heartRate);
  }

  @override
  void didReceiveMagnetometer(
      sdk.Device_ device, int timestamp, double mx, double my, double mz) {
    delegate?.didReceiveMagnetometer(this, timestamp, mx, my, mz);
  }

  @override
  void didReceiveMessage(sdk.Device_ device, String process, String message) {
    delegate?.didReceiveMessage(this, process, message);
  }

  @override
  void didReceiveOrientation(sdk.Device_ device, int timestamp, double roll,
      double pitch, double yaw) {
    delegate?.didReceiveOrientation(this, timestamp, roll, pitch, yaw);
  }

  @override
  void didReceivePastAccelerometer(
      sdk.Device_ device, int timestamp, double ax, double ay, double az) {
    delegate?.didReceivePastAccelerometer(this, timestamp, ax, ay, az);
  }

  @override
  void didReceivePastActivity(
      sdk.Device_ device, int timestamp, sdk.ActivityType_ activity) {
    delegate?.didReceivePastActivity(this, timestamp, activity);
  }

  @override
  void didReceivePastBodyPosition(
      sdk.Device_ device, int timestamp, sdk.BodyPosition_ bodyPosition) {
    delegate?.didReceivePastBodyPosition(this, timestamp, bodyPosition);
  }

  @override
  void didReceivePastECG(sdk.Device_ device, int timestamp, double value) {
    delegate?.didReceivePastECG(this, timestamp, value);
  }

  @override
  void didReceivePastGyroscope(
      sdk.Device_ device, int timestamp, double qx, double qy, double qz) {
    delegate?.didReceivePastGyroscope(this, timestamp, qx, qy, qz);
  }

  @override
  void didReceivePastHeartRate(
      sdk.Device_ device, int timestamp, int heartRate) {
    delegate?.didReceivePastHeartRate(this, timestamp, heartRate);
  }

  @override
  void didReceivePastMagnetometer(
      sdk.Device_ device, int timestamp, double mx, double my, double mz) {
    delegate?.didReceivePastMagnetometer(this, timestamp, mx, my, mz);
  }

  @override
  void didReceivePastOrientation(sdk.Device_ device, int timestamp, double roll,
      double pitch, double yaw) {
    delegate?.didReceivePastOrientation(this, timestamp, roll, pitch, yaw);
  }

  @override
  void didReceivePastPressure(sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceivePastPressure(this, timestamp, value);
  }

  @override
  void didReceivePastQuaternion(sdk.Device_ device, int timestamp, double qw,
      double qx, double qy, double qz) {
    delegate?.didReceivePastQuaternion(this, timestamp, qw, qx, qy, qz);
  }

  @override
  void didReceivePastRespiration(
      sdk.Device_ device, int timestamp, double value) {
    delegate?.didReceivePastRespiration(this, timestamp, value);
  }

  @override
  void didReceivePastRespirationRate(
      sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceivePastRespirationRate(this, timestamp, value);
  }

  @override
  void didReceivePastRr(sdk.Device_ device, int timestamp, int rr) {
    delegate?.didReceivePastRr(this, timestamp, rr);
  }

  @override
  void didReceivePastSignalQuality(
      sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceivePastSignalQuality(this, timestamp, value);
  }

  @override
  void didReceivePastSkinTemperature(
      sdk.Device_ device, int timestamp, double value) {
    delegate?.didReceivePastSkinTemperature(this, timestamp, value);
  }

  @override
  void didReceivePastSoundFeatures(
      sdk.Device_ device, int timestamp, List<double?> values) {
    delegate?.didReceivePastSoundFeatures(
        this, timestamp, values.map((e) => e!).toList());
  }

  @override
  void didReceivePastSoundVolume(
      sdk.Device_ device, int timestamp, int soundVolume) {
    delegate?.didReceivePastSoundVolume(this, timestamp, soundVolume);
  }

  @override
  void didReceivePastSteps(sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceivePastSteps(this, timestamp, value);
  }

  @override
  void didReceivePressure(sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceivePressure(this, timestamp, value);
  }

  @override
  void pressureWearStateDidChange(
      sdk.Device_ device, sdk.WearState_ wearState) {
    delegate?.didReceivePressureWearState(this, wearState);
  }

  @override
  void didReceiveQuaternion(sdk.Device_ device, int timestamp, double qw,
      double qx, double qy, double qz) {
    delegate?.didReceiveQuaternion(this, timestamp, qw, qx, qy, qz);
  }

  @override
  void didReceiveRespiration(sdk.Device_ device, int timestamp, double value) {
    delegate?.didReceiveRespiration(this, timestamp, value);
  }

  @override
  void didReceiveRespirationRate(sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceiveRespirationRate(this, timestamp, value);
  }

  @override
  void didReceiveRr(sdk.Device_ device, int timestamp, int rr) {
    delegate?.didReceiveRr(this, timestamp, rr);
  }

  @override
  void didReceiveSignalQuality(sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceiveSignalQuality(this, timestamp, value);
  }

  @override
  void didReceiveSkinTemperature(
      sdk.Device_ device, int timestamp, double value) {
    delegate?.didReceiveSkinTemperature(this, timestamp, value);
  }

  @override
  void didReceiveSoundFeatures(
      sdk.Device_ device, int timestamp, List<double?> values) {
    delegate?.didReceiveSoundFeatures(
        this, timestamp, values.map((e) => e!).toList());
  }

  @override
  void didReceiveSoundVolume(sdk.Device_ device, int timestamp, int value) {
    delegate?.didReceiveSoundVolume(this, timestamp, value);
  }

  @override
  void didReceiveSteps(sdk.Device_ device, int timestamp, int steps) {
    delegate?.didReceiveSteps(this, timestamp, steps);
  }

  @override
  void didReceiveUnsynchronizedSize(
      sdk.Device_ device, int unsynchronizedSize, double syncBytesPerSecond) {
    delegate?.didReceiveUnsynchronizedSize(
        this, unsynchronizedSize, syncBytesPerSecond);
  }

  @override
  void syncStateDidChange(sdk.Device_ device, sdk.SyncState_ state) {
    delegate?.syncStateDidChange(this, state);
  }

  @override
  void wearStateDidChange(sdk.Device_ device, sdk.WearState_ wearState) {
    delegate?.wearStateDidChange(this, wearState);
  }
}

abstract class DeviceDelegate {
  void didConnect(Device device);

  void didDisconnect(Device device, DisconnectReason reason);

  void didReceiveBatteryLevel(Device device, int stateOfCharge);

  void didReceiveECG(Device device, int timestamp, double value);

  void didReceiveRespiration(Device device, int timestamp, double value);

  void didReceiveSkinTemperature(Device device, int timestamp, double value);

  void didReceiveHeartRate(Device device, int timestamp, int heartRate);

  void didReceiveRr(Device device, int timestamp, int rr);

  void didReceiveSteps(Device device, int timestamp, int steps);

  void didReceiveActivity(Device device, int timestamp, ActivityType activity);

  void didReceiveSoundVolume(Device device, int timestamp, int value);

  void didReceiveSoundFeatures(
      Device device, int timestamp, List<double> values);

  void didReceivePressure(Device device, int timestamp, int value);

  void didReceivePressureWearState(Device device, WearState wearState);

  void didReceiveCommand(Device device);

  void didReceiveError(String error);

  void didReceiveMessage(Device device, String process, String message);

  void didDetectUserEvent(Device device, int timestamp);

  void didDetectExercise(Device device, Exercise exercise);

  void didReceiveRespirationRate(Device device, int timestamp, int value);

  void didReceiveSignalQuality(Device device, int timestamp, int value);

  void syncStateDidChange(Device device, SyncState state);

  void didReceivePastECG(Device device, int timestamp, double value);

  void didReceivePastRespiration(Device device, int timestamp, double value);

  void didReceivePastSkinTemperature(
      Device device, int timestamp, double value);

  void didReceivePastHeartRate(Device device, int timestamp, int heartRate);

  void didReceivePastRr(Device device, int timestamp, int rr);

  void wearStateDidChange(Device device, WearState wearState);

  void didReceiveUnsynchronizedSize(
      Device device, int unsynchronizedSize, double syncBytesPerSecond);

  void didReceivePastRespirationRate(Device device, int timestamp, int value);

  void didReceivePastActivity(
      Device device, int timestamp, ActivityType activity);

  void didReceivePastSteps(Device device, int timestamp, int value);

  void didReceivePastSoundVolume(Device device, int timestamp, int soundVolume);

  void didReceivePastSoundFeatures(
      Device device, int timestamp, List<double> values);

  void didReceivePastPressure(Device device, int timestamp, int value);

  void didReceivePastSignalQuality(Device device, int timestamp, int value);

  void didReceivePastUserEvent(Device device, int timestamp);

  void didReceivePastMagnetometer(
      Device device, int timestamp, double mx, double my, double mz);

  void didReceiveMagnetometer(
      Device device, int timestamp, double mx, double my, double mz);

  void didReceivePastGyroscope(
      Device device, int timestamp, double qx, double qy, double qz);

  void didReceiveGyroscope(
      Device device, int timestamp, double qx, double qy, double qz);

  void didReceivePastAccelerometer(
      Device device, int timestamp, double ax, double ay, double az);

  void didReceiveAccelerometer(
      Device device, int timestamp, double ax, double ay, double az);

  void didReceivePastQuaternion(
      Device device, int timestamp, double qw, double qx, double qy, double qz);

  void didReceiveQuaternion(
      Device device, int timestamp, double qw, double qx, double qy, double qz);

  void didReceivePastOrientation(
      Device device, int timestamp, double roll, double pitch, double yaw);

  void didReceiveOrientation(
      Device device, int timestamp, double roll, double pitch, double yaw);

  void didReceivePastBodyPosition(
      Device device, int timestamp, BodyPosition bodyPosition);

  void didReceiveBodyPosition(
      Device device, int timestamp, BodyPosition bodyPosition);
}
