import 'package:pigeon/pigeon.dart';

@ConfigurePigeon(PigeonOptions(
  dartOut: 'lib/src/platform_aidlab_sdk.dart',
  kotlinOut:
      'android/src/main/kotlin/com/aidlab/aidlab_sdk/PlatformAidlabSdk.kt',
  kotlinOptions: KotlinOptions(package: 'com.aidlab.aidlab_sdk'),
  swiftOut: 'ios/Classes/Flutter/PlatformAidlabSdk.swift',
))
@HostApi()
abstract class AidlabManager_ {
  void scan(ScanMode_ mode);

  void stopScan();

  void connect(Device_ device);

  void collect(Device_ device, List<FlutterDataType_> dataTypes,
      List<FlutterDataType_> dataTypesToStore);

  void disconnect(Device_ device);
}

class FlutterDataType_ {
  DataType_? signal;
}

@FlutterApi()
abstract class AidlabManagerDelegate_ {
  void onBluetoothStarted();

  void onDeviceScanStarted();

  void onDeviceScanStopped();

  void onScanFailed(int errorCode);

  void didDiscover(Device_ device, int rssi);
}

enum SyncState_ { Start, End, Stop, Empty }

enum WearState_ { PlacedProperly, PlacedUpsideDown, Loose, Detached, Unknown }

enum EcgFiltrationMethod_ { normal, aggressive }

enum DisconnectReason_ {
  Timeout,
  DeviceDisconnected,
  AppDisconnected,
  SdkOutdated,
  UnknownError,
}

enum DataType_ {
  ECG,
  RESPIRATION,
  SKIN_TEMPERATURE,
  MOTION,
  RESERVED_0, // Was BATTERY. Enabled by default since SDK 1.6.0
  ACTIVITY,
  ORIENTATION,
  STEPS,
  HEART_RATE,
  RESERVED_1, // Was HEALTH_THERMOMETHER. No longer in use. Use SKIN_TEMPERATURE instead.
  SOUND_VOLUME,
  RR,
  PRESSURE, // Supported since Firmware 3.0.0. No longer available as characteristic.
  SOUND_FEATURES,
  RESPIRATION_RATE,
  BODY_POSITION;
}

enum ScanMode_ {
  lowPower,
  aggressive,
}

@FlutterApi()
abstract class DeviceDelegate_ {
  void didConnect(Device_ device);

  void didDisconnect(Device_? device, DisconnectReason_ reason);

  void wearStateDidChange(Device_ device, WearState_ wearState);

  void didReceiveECG(Device_ device, int timestamp, double value);

  void didReceiveRespiration(Device_ device, int timestamp, double value);

  void didReceiveRespirationRate(Device_ device, int timestamp, int value);

  void didReceiveBatteryLevel(Device_ device, int stateOfCharge);

  void didReceiveSkinTemperature(Device_ device, int timestamp, double value);

  void didReceiveAccelerometer(
      Device_ device, int timestamp, double ax, double ay, double az);

  void didReceiveGyroscope(
      Device_ device, int timestamp, double qx, double qy, double qz);

  void didReceiveMagnetometer(
      Device_ device, int timestamp, double mx, double my, double mz);

  void didReceiveQuaternion(Device_ device, int timestamp, double qw, double qx,
      double qy, double qz);

  void didReceiveOrientation(
      Device_ device, int timestamp, double roll, double pitch, double yaw);

  void didReceiveBodyPosition(
      Device_ device, int timestamp, BodyPosition_ bodyPosition);

  void didReceiveActivity(
      Device_ device, int timestamp, ActivityType_ activity);

  void didReceiveSteps(Device_ device, int timestamp, int steps);

  void didReceiveHeartRate(Device_ device, int timestamp, int heartRate);

  void didReceiveRr(Device_ device, int timestamp, int rr);

  void didDetectExercise(Device_ device, Exercise_ exercise);

  void didReceiveSoundVolume(Device_ device, int timestamp, int value);

  void didReceiveSoundFeatures(
      Device_ device, int timestamp, List<double> values);

  void didReceivePressure(Device_ device, int timestamp, int value);

  void pressureWearStateDidChange(Device_ device, WearState_ wearState);

  void didReceiveSignalQuality(Device_ device, int timestamp, int value);

  void didDetectUserEvent(Device_ device, int timestamp);

  //-- System

  void didReceiveCommand(Device_ device);

  void didReceiveError(String error);

  void didReceiveMessage(Device_ device, String process, String message);

  void syncStateDidChange(Device_ device, SyncState_ state);

  void didReceiveUnsynchronizedSize(
      Device_ device, int unsynchronizedSize, double syncBytesPerSecond);

  //-- Past

  void didReceivePastECG(Device_ device, int timestamp, double value);

  void didReceivePastRespiration(Device_ device, int timestamp, double value);

  void didReceivePastSkinTemperature(
      Device_ device, int timestamp, double value);

  void didReceivePastHeartRate(Device_ device, int timestamp, int heartRate);

  void didReceivePastRr(Device_ device, int timestamp, int rr);

  void didReceivePastRespirationRate(Device_ device, int timestamp, int value);

  void didReceivePastActivity(
      Device_ device, int timestamp, ActivityType_ activity);

  void didReceivePastSteps(Device_ device, int timestamp, int value);

  void didReceivePastSoundVolume(
      Device_ device, int timestamp, int soundVolume);

  void didReceivePastSoundFeatures(
      Device_ device, int timestamp, List<double> values);

  void didReceivePastAccelerometer(
      Device_ device, int timestamp, double ax, double ay, double az);

  void didReceivePastGyroscope(
      Device_ device, int timestamp, double qx, double qy, double qz);

  void didReceivePastMagnetometer(
      Device_ device, int timestamp, double mx, double my, double mz);

  void didReceivePastQuaternion(Device_ device, int timestamp, double qw,
      double qx, double qy, double qz);

  void didReceivePastOrientation(
      Device_ device, int timestamp, double roll, double pitch, double yaw);

  void didReceivePastBodyPosition(
      Device_ device, int timestamp, BodyPosition_ bodyPosition);

  void didReceivePastPressure(Device_ device, int timestamp, int value);

  void didDetectPastUserEvent(Device_ device, int timestamp);

  void didReceivePastSignalQuality(Device_ device, int timestamp, int value);
}

class Device_ {
  String? firmwareRevision;
  String? hardwareRevision;
  String? serialNumber;
  String? name;
  String? manufacturerName;
  late String address;
}

enum BodyPosition_ { undefined, prone, supine, leftSide, rightSide }

enum Exercise_ {
  PushUp,
  Jump,
  SitUp,
  Burpee,
  PullUp,
  Squat,
  PlankStart,
  PlankEnd
}

enum ActivityType_ {
  Unspecific,
  Automotive,
  Walking,
  Running,
  Cycling,
  Still,
}
